/*
 * Copyright 2021 Nilanjan Daw, Synerg Lab, Department of CSE, IIT Bombay
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package HpdosClient.MessageFormat;

import hpdos.grpc.Ack;
import hpdos.grpc.Nack;
import hpdos.grpc.Packet;
import hpdos.grpc.Response;

import java.util.ArrayList;

public class ResponseBuilder {

    public static Ack buildAck(int version, int dataSize, String key, long crc, String value) {
        Ack.Builder ack = Ack.newBuilder();
        ack.setKey(key);
        ack.setVersion(version);
        ack.setDataSize(dataSize);
        ack.setCrc(crc);
        ack.setValue(value);
        return ack.build();
    }

    public static Nack buildNack(int version, String key) {
        Nack.Builder nack = Nack.newBuilder();
        nack.setKey(key);
        nack.setVersion(version);
        return nack.build();
    }

    public static Response buildResponsePacket(int operationType, int status,
                                               Ack ack,
                                               Nack nack) {
        Response.Builder response = Response.newBuilder();
        response.setOperationType(operationType);
        response.setStatus(status);
        if (ack != null)
            response.setAck(ack);
        else
            response.clearAck();
        if (nack != null)
            response.setNack(nack);
        else
            response.clearNack();
        return response.build();
    }

    public static Packet buildPacket(ArrayList<Response> responses) {
        Packet.Builder packet = Packet.newBuilder();
        packet.setPacketType(MessageConstants.PACKET_METADATA_RESPONSE);
        packet.addAllResponse(responses);
        return packet.build();
    }

    public static Packet buildPacket(Response response) {
        Packet.Builder packet = Packet.newBuilder();
        packet.setPacketType(MessageConstants.PACKET_METADATA_RESPONSE);
        packet.addResponse(response);
        return packet.build();
    }
}
