/*
 * Copyright 2021 Nilanjan Daw, Synerg Lab, Department of CSE, IIT Bombay
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package hpdos.handler;

import com.google.common.base.Stopwatch;
import hpdos.grpc.*;
import hpdos.message.MessageConstants;
import hpdos.message.ResponseBuilder;
import io.grpc.stub.StreamObserver;

public class ReplicateHandler extends ReplicationServiceGrpc.ReplicationServiceImplBase {
    private final IOHandler ioHandler;
    public ReplicateHandler(IOHandler ioHandler) {
        this.ioHandler = ioHandler;
    }
    @Override
    public void replicateMetadata(ReplicationRequest replicationRequest, StreamObserver<ReplicationResponse> responseObserver) {
        ReplicationResponse responsePacket = null;
//        System.out.println("Replication request " + replicationRequest);
        Stopwatch stopwatch = Stopwatch.createUnstarted();
        stopwatch.start();
        if (replicationRequest.getPacketType() == MessageConstants.PACKET_METADATA_REQUEST) {
            if (replicationRequest.getRequestList().size() > 1)
                System.out.println("REQUEST BATCHING");
            for (Request request: replicationRequest.getRequestList()) {
                if (request.getOperationType() == MessageConstants.METADATA_CREATE) {
                    Response response = ioHandler.create(request);
                    responsePacket = ResponseBuilder.buildReplicationResponse(response);
                } else if (request.getOperationType() == MessageConstants.METADATA_UPDATE) {
                    Response response = ioHandler.update(request);
                    responsePacket = ResponseBuilder.buildReplicationResponse(response);
                } else if (request.getOperationType() == MessageConstants.METADATA_DELETE) {
                    Response response = ioHandler.delete(request);
                    responsePacket = ResponseBuilder.buildReplicationResponse(response);
                }
//                System.out.println(responsePacket);
                responseObserver.onNext(responsePacket);
            }
            responseObserver.onCompleted();
        }
        stopwatch.stop();
//        System.out.println("Replicate Handler " + stopwatch);
    }

}
