/*
 * Copyright 2021 Nilanjan Daw, Synerg Lab, Department of CSE, IIT Bombay
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package hpdos.message;

import hpdos.grpc.Packet;
import hpdos.grpc.ReplicationRequest;
import hpdos.grpc.Request;

import java.util.ArrayList;


public class RequestBuilder {
    public static Request buildRequest(int operationType, int version,
                                                         int dataSize, String key, int crc, String value) {
        Request.Builder request = Request.newBuilder();
        request.setOperationType(operationType);
        request.setVersion(version);
        request.setDataSize(dataSize);
        request.setKey(key);
        request.setCrc(crc);
        request.setValue(value);
        return request.build();
    }

    public static Packet buildPacket(ArrayList<Request> requests) {
        Packet.Builder packet = Packet.newBuilder();
        packet.setPacketType(MessageConstants.PACKET_METADATA_REQUEST);
        packet.addAllRequest(requests);
        return packet.build();
    }

    public static ReplicationRequest buildReplicationRequest(ArrayList<Request> requests) {
        ReplicationRequest.Builder builder = ReplicationRequest.newBuilder();
        builder.setPacketType(MessageConstants.PACKET_METADATA_REQUEST);
        builder.addAllRequest(requests);
        return builder.build();
    }
    
}
